package com.miaozhen.mzanalytics_mybmw;

import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.net.Uri;
import android.util.Log;

import com.miaozhen.sitesdk.api.MzSiteSDK;
import com.miaozhen.sitesdk.util.Logger;

import org.json.JSONArray;
import org.json.JSONObject;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class EventUtils {
    /**
     * 事件包括：初始化、页面、基础、高级事件、心跳、deeplink。
     */

    static MzSiteSDK mzSiteSDK;

    public static void init(Context context,JSONObject object) {
        try {
            if (mzSiteSDK == null) {
                initInstance(context);
            }
            String appkey = object.optString("appkey");
            mzSiteSDK.init(appkey);
        } catch (Throwable e) {
            e.printStackTrace();
        }

    }

    public static void initInstance(Context context) {
        try {
            mzSiteSDK = MzSiteSDK.getInstance(context);
        } catch (Throwable e) {
            e.printStackTrace();
        }

    }


    public static void trackPageView(Context context, JSONObject pageObject) {
        if (mzSiteSDK == null) {
            initInstance(context);
        }
        try {

            String title = pageObject.optString("dt");
            JSONObject object = pageObject.optJSONObject("CustomDimMetrics");
            mzSiteSDK.trackPageView(title, object);

        } catch (Throwable e) {
            e.printStackTrace();
        }
    }

    public static void trackEvent(Context context, JSONObject object) {
        if (mzSiteSDK == null) {
            initInstance(context);
        }
        //基础事件
        try {
            String eventClassifyStr = object.optString("ec");
            String eventActionStr = object.optString("ea");
            String eventTagStr = object.optString("el");
            String eventPriceStr = object.optString("ev");
            JSONObject eventObject = object.optJSONObject("CustomDimMetrics");
            mzSiteSDK.trackEvent(eventClassifyStr, eventActionStr, eventTagStr, eventPriceStr, eventObject);
        } catch (Throwable e) {
            e.printStackTrace();
        }


    }

    public static void trackSiteCustomEvent(Context context, JSONObject object) {
        if (mzSiteSDK == null) {
            initInstance(context);
        }

        //自定义事件和高级事件监测
        try {
            JSONObject customEvent = object.optJSONObject("CustomEvent");
            JSONObject customDimMetrics = object.optJSONObject("CustomDimMetrics");
            mzSiteSDK.trackSiteCustomEvent(customEvent, customDimMetrics);
        } catch (Throwable e) {
            e.printStackTrace();
        }

    }

    public static void trackPulse(Context context) {
        try {
            if (mzSiteSDK == null) {
                initInstance(context);
            }
            mzSiteSDK.trackPulse();
        } catch (Throwable e) {
            e.printStackTrace();
        }
    }

    public static void deepLinkEvent(Context context, JSONObject object, Activity activity) {
        try {
            Intent intent = new Intent();
            String deeplinkStr = object.optString("linUrl");
            if (deeplinkStr == null || deeplinkStr.equals("")) {
                Logger.d("handleOpenURL deeplink参数为空");
                return;
            }
            Uri uriData = Uri.parse(deeplinkStr);
            intent.setData(uriData);
            mzSiteSDK.trackDeeplink(intent,activity);
        } catch (Throwable e) {
            e.printStackTrace();
        }

    }

    public static String getCid(Context context) {
        String result = "";
        try {
            if (mzSiteSDK == null) {
                initInstance(context);
            }
            result = mzSiteSDK.getCid();
        } catch (Throwable e) {
            e.printStackTrace();
        }
        return result;
    }

    public static String getOaid(Context context) {
        String result = "";
        try {
            if (mzSiteSDK == null) {
                initInstance(context);
            }
            result = mzSiteSDK.getOaid();
        } catch (Throwable e) {
            e.printStackTrace();
        }
        return result;
    }


    public static void setCd(Context context, String key, String value) {
        try {
            if (mzSiteSDK == null) {
                initInstance(context);
            }
            mzSiteSDK.setCd(key, value);
        } catch (Throwable e) {
            e.printStackTrace();
        }
    }

    public static void setCm(Context context, String key, String value) {
        try {
            if (mzSiteSDK == null) {
                initInstance(context);
            }
            mzSiteSDK.setCm(key, value);
        } catch (Throwable e) {
            e.printStackTrace();
        }
    }

    public static void deleteCd(Context context, String key) {
        try {
            if (mzSiteSDK == null) {
                initInstance(context);
            }
            mzSiteSDK.deleteCd(key);
        } catch (Throwable e) {
            e.printStackTrace();
        }
    }

    public static void deleteCm(Context context, String key) {
        try {
            if (mzSiteSDK == null) {
                initInstance(context);
            }
            mzSiteSDK.deleteCm(key);
        } catch (Throwable e) {
            e.printStackTrace();
        }
    }

    public static Map<String, String> getCdAndCm(Context context) {
        try {
            if (mzSiteSDK == null) {
                initInstance(context);
            }
            return mzSiteSDK.getCdAndCm();
        } catch (Throwable e) {
            e.printStackTrace();
            return new HashMap<>();
        }
    }

    public static boolean isAutoTrackpage(Context context) {
        boolean result = false;
        try {
            if (mzSiteSDK == null) {
                initInstance(context);
            }
            result = mzSiteSDK.isAutoTrackpage();
        } catch (Throwable e) {
            e.printStackTrace();

        }
        return result;
    }

    public static List<String> getManualtrackpage(Context context) {
        List result = new ArrayList();
        try {
            if (mzSiteSDK == null) {
                initInstance(context);
            }
            List tmpList = mzSiteSDK.getBlackList();
            result.addAll(tmpList);
        } catch (Throwable e) {
            e.printStackTrace();

        }
        return result;
    }

    public static void setLogEnabled(Context context, boolean isDebug) {
        try {
            if (mzSiteSDK == null) {
                initInstance(context);
            }
            mzSiteSDK.setLogEnabled(isDebug);
        } catch (Throwable e) {
            e.printStackTrace();

        }
    }
    public static void enableExceptionService(Context context, boolean isDebug) {
        try {
            if (mzSiteSDK == null) {
                initInstance(context);
            }
            mzSiteSDK.enableExceptionService(isDebug);
        } catch (Throwable e) {
            e.printStackTrace();

        }
    }
    public static void trackException(Context context, JSONObject object) {
        try {
            if (mzSiteSDK == null) {
                initInstance(context);
            }

            Map map = new HashMap();
            map.put("name",object.optString("name"));
            map.put("customerror",object.optString("customerror"));
            map.put("reason",object.optString("reason"));
            map.put("stack",object.optString("stack").toString());
            mzSiteSDK.trackFlutterException(map);
        } catch (Throwable e) {
            e.printStackTrace();
        }

    }
    public static void trackException(Context context, Throwable throwable) {
        try {
            if (mzSiteSDK == null) {
                initInstance(context);
            }
            mzSiteSDK.trackException(throwable);
        } catch (Throwable e) {
            e.printStackTrace();
        }

    }


}
