import 'dart:async';

import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:mzanalytics_mybmw/mzanalytics_mybmw.dart';

Future<void> main() async {
  runApp(CrashTestApp());
}

class CrashTestApp extends StatefulWidget {
  @override
  _MyAppState createState() => _MyAppState();
}

class _MyAppState extends State<CrashTestApp> {
  @override
  void initState() {
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return MaterialApp(
      home: MainScaffold(),
    );
  }
}

class MainScaffold extends StatelessWidget {
  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(title: const Text('Crash Test')),
      body: SingleChildScrollView(
        child: Column(
          children: [
            const Center(child: Text('Trigger an action:\n')),
            ElevatedButton(
              child: const Text('Flutter error :Draw the crash'),
              onPressed: () => SecondaryScaffold.openSecondaryScaffold(context),
            ),

            ElevatedButton(
              child: const Text('Flutter error : Scaffold.of()'),
              onPressed: () => Scaffold.of(context).showBottomSheet<dynamic>(
                  (context) => const Text('Scaffold error')),
            ),
            ElevatedButton(
              child: const Text('Dart: StateError'),
              onPressed: () => throw StateError('Throws StateError'),
            ),
            ElevatedButton(
              child: const Text('Dart: throw onPressed'),
              // Warning : not captured if a debugger is attached
              // https://github.com/flutter/flutter/issues/48972
              onPressed: () => throw Exception('Throws onPressed'),
            ),
            ElevatedButton(
              child: const Text('Dart: assert'),
              onPressed: () {
                // Only relevant in debug builds
                // Warning : not captured if a debugger is attached
                // https://github.com/flutter/flutter/issues/48972
                Timer(Duration(seconds: 3), () {
                  assert(false, 'assert failure');
                  print("Yeah, this line is printed after 3 seconds");
                });
              },
            ),
            // Calling the SDK with an appRunner will handle errors from Futures
            // in SDKs runZonedGuarded onError handler

            ElevatedButton(
              child: const Text('Dart: Fail in compute'),
              onPressed: () => throw compute(loop, 10),
            ),
            ElevatedButton(
              child: const Text('Native: Manual exception'),
              onPressed: () {
                MzanalyticsMybmw.MZNativeErrorTest();
              },
            ),
            ElevatedButton(
              child: const Text('Flutter: Manual exception'),
              onPressed: () {
                Future.delayed(Duration(seconds: 3), () {
                  try {
                    var s = ['1'];
                    print(s[1]);
                  } catch (e, stack) {
                    Map exceptionMap = {
                      "name": e.toString(),
                      "reason": 'Manual exception',
                      "stack": stack.toString(),
                      "customerror": "0"
                    };

                    MzanalyticsMybmw.trackException(exceptionMap);
                  }
                });
              },
            ),
          ],
        ),
      ),
    );
  }
}

/// compute can only take a top-level function, but not instance or static methods.
// Top-level functions are functions declared not inside a class and not inside another function
int loop(int val) {
  var count = 0;
  for (var i = 1; i <= val; i++) {
    count += i;
  }

  throw StateError('from a compute isolate $count');
}

class SecondaryScaffold extends StatelessWidget {
  static Future<void> openSecondaryScaffold(BuildContext context) {
    return Navigator.push(
      context,
      MaterialPageRoute<void>(
        settings:
            const RouteSettings(name: 'SecondaryScaffold', arguments: 'foobar'),
        builder: (context) {
          return SecondaryScaffold();
        },
      ),
    );
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        title: const Text('SecondaryScaffold'),
      ),
      body: Center(
        child: Column(
          children: [
            const Text(
              'You have added a navigation event '
              'to the crash reports breadcrumbs.',
            ),
            MaterialButton(
              child: const Text('Go back'),
              height: -10,
              onPressed: () {
                Navigator.pop(context);
              },
            ),
          ],
        ),
      ),
    );
  }
}
