import 'package:flutter/material.dart';
import 'dart:async';

import 'package:flutter/foundation.dart';
import 'package:flutter/services.dart';
import 'package:mzanalytics_mybmw/mzanalytics_mybmw.dart';
import 'package:mzanalytics_mybmw/mzanalytics_bugService.dart';

void main() => Mzanalytics_bugService.exceptionListener(
      () => runApp(MyApp()),
    );

class MyApp extends StatefulWidget {
  @override
  _MyAppState createState() => _MyAppState();
}

class _MyAppState extends State<MyApp> {
  String _cid = 'Unknown';
  String _oaid = 'Unknown';
  String _myevent = '自定义事件';

  @override
  void initState() {
    super.initState();
  }

  // Platform messages are asynchronous, so we initialize in an async method.

  void getcid() async {
    // callback function

    String cid = await MzanalyticsMybmw.getCid();

    setState(() {
      _cid = cid;
    });
  }

  @override
  Widget build(BuildContext context) {
    MzanalyticsMybmw.logEnabled(true);
    return MaterialApp(
        // navigatorObservers: [MZNavigationHistoryObserver()],
        home: const MainScaffold());
  }
}

class MainScaffold extends StatelessWidget {
  const MainScaffold({
    Key key,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(title: const Text('Sentry Flutter Example')),
      body: SingleChildScrollView(
        child: Column(
          children: [
            const Center(child: Text('Trigger an action:\n')),
            ElevatedButton(
              child: const Text('Open another Scaffold'),
              onPressed: () {
                MzanalyticsMybmw.appid("2222");
                MzanalyticsMybmw.enableExceptionService(true);
              },
            ),
            ElevatedButton(
              child: const Text('Dart: StateError'),
              onPressed: () => throw StateError('Throws StateError'),
            ),
            ElevatedButton(
              child: const Text('Flutter error : Scaffold.of()'),
              onPressed: () => Scaffold.of(context).showBottomSheet<dynamic>(
                  (context) => const Text('Scaffold error')),
            ),
            ElevatedButton(
              child: const Text('Dart: throw onPressed'),
              // Warning : not captured if a debugger is attached
              // https://github.com/flutter/flutter/issues/48972
              onPressed: () => throw Exception('Throws onPressed'),
            ),
            ElevatedButton(
              child: const Text('Dart: assert'),
              onPressed: () {
                // Only relevant in debug builds
                // Warning : not captured if a debugger is attached
                // https://github.com/flutter/flutter/issues/48972
                assert(false, 'assert failure');
              },
            ),
            // Calling the SDK with an appRunner will handle errors from Futures
            // in SDKs runZonedGuarded onError handler

            ElevatedButton(
              child: const Text('Dart: OutOfMemoryError'),
              onPressed: () => throw OutOfMemoryError(),
            ),
            ElevatedButton(
              child: const Text('Dart: StackOverflowError'),
              onPressed: () => Future.delayed(Duration(milliseconds: 100),
                  () => throw StackOverflowError()),
            ),
            ElevatedButton(
                child: const Text('Dart: Web request'), onPressed: () {}),
            ElevatedButton(
              child: const Text('Record print() as breadcrumb'),
              onPressed: () {
                print('A print breadcrumb');
              },
            ),

            const CocoaExample(),
          ],
        ),
      ),
    );
  }
}

class SecondaryScaffold {}

Future<void> execute(String method) async {}

class CocoaExample extends StatelessWidget {
  const CocoaExample({Key key}) : super(key: key);

  final channel = const MethodChannel('example.flutter.sentry.io');

  @override
  Widget build(BuildContext context) {
    return Column(
      children: [
        ElevatedButton(
          child: const Text('Swift fatalError'),
          onPressed: () async {
            await channel.invokeMethod<void>('fatalError');
          },
        ),
        ElevatedButton(
          child: const Text('Swift Capture NSException'),
          onPressed: () async {
            await channel.invokeMethod<void>('capture');
          },
        ),
        ElevatedButton(
          child: const Text('Swift Capture message'),
          onPressed: () async {
            await channel.invokeMethod<void>('capture_message');
          },
        ),
        ElevatedButton(
          child: const Text('Objective-C Throw unhandled exception'),
          onPressed: () async {
            await channel.invokeMethod<void>('throw');
          },
        ),
        ElevatedButton(
          child: const Text('Objective-C SEGFAULT'),
          onPressed: () async {
            await channel.invokeMethod<void>('crash');
          },
        ),
      ],
    );
  }
}
