import 'dart:async';
import 'package:flutter/services.dart';
import 'package:mzanalytics_mybmw/mzanalytics_log.dart';

///Mzanalytics_android
class MzanalyticsAndroid {
  static const MethodChannel _channel =
      const MethodChannel('mzanalytics_mybmw');

  ///initEvent
  static Future<void> initEvent(String aid) async {
    try {
      _channel.invokeMethod('initEvent', aid);
    } catch (e, stack) {
      MzanalyticsLog.p(e.toString() + stack.toString());
    }
  }

  ///trackEvent
  static Future<void> trackEvent(var parameter) async {
    try {
      _channel.invokeMethod('trackEvent', parameter);
    } catch (e, stack) {
      MzanalyticsLog.p(e.toString() + stack.toString());
    }
  }

  ///trackPageView
  static Future<void> trackPageView(var parameter) async {
    try {
      _channel.invokeMethod('trackPageView', parameter);
    } catch (e, stack) {
      MzanalyticsLog.p(e.toString() + stack.toString());
    }
  }

  ///trackSiteCustomEvent
  static Future<void> trackSiteCustomEvent(var parameter) async {
    try {
      _channel.invokeMethod('trackSiteCustomEvent', parameter);
    } catch (e, stack) {
      MzanalyticsLog.p(e.toString() + stack.toString());
    }
  }

  ///trackPulse
  static Future<void> trackPulse() async {
    try {
      _channel.invokeMethod('trackPulse');
    } catch (e, stack) {
      MzanalyticsLog.p(e.toString() + stack.toString());
    }
  }

  ///handleOpenURL
  static Future<void> handleOpenURL(String url) async {
    try {
      _channel.invokeMethod('handleOpenURL', url);
    } catch (e, stack) {
      MzanalyticsLog.p(e.toString() + stack.toString());
    }
  }

  ///getCid
  static Future<String?> getCid() async {
    try {
      return _channel.invokeMethod('getCid');
    } catch (e, stack) {
      MzanalyticsLog.p(e.toString() + stack.toString());
    }
    return "";
  }

  ///setCd
  static Future<void> setCd(var key, var value) async {
    try {
      return _channel.invokeMethod(
          'setCd', <String, dynamic>{'cd_key': key, "cd_value": value});
    } catch (e, stack) {
      MzanalyticsLog.p(e.toString() + stack.toString());
    }
  }

  ///setCm
  static Future<void> setCm(var key, var value) async {
    try {
      return _channel.invokeMethod(
          'setCm', <String, dynamic>{'cm_key': key, "cm_value": value});
    } catch (e, stack) {
      MzanalyticsLog.p(e.toString() + stack.toString());
    }
  }

  ///deleteCd
  static Future<void> deleteCd(var key) async {
    try {
      return _channel
          .invokeMethod('deleteCd', <String, dynamic>{'cd_key': key});
    } catch (e, stack) {
      MzanalyticsLog.p(e.toString() + stack.toString());
    }
  }

  ///deleteCm
  static Future<void> deleteCm(var key) async {
    try {
      return _channel
          .invokeMethod('deleteCm', <String, dynamic>{'cm_key': key});
    } catch (e, stack) {
      MzanalyticsLog.p(e.toString() + stack.toString());
    }
  }

  ///getCdAndCm
  static Future<Map?> getCdAndCm() async {
    try {
      return _channel.invokeMethod('getCdAndCm');
    } catch (e, stack) {
      MzanalyticsLog.p(e.toString() + stack.toString());
    }
    return Map();
  }

  ///isAutoTrackpage
  static Future<bool?> isAutoTrackpage() async {
    try {
      return _channel.invokeMethod('isAutoTrackpage');
    } catch (e, stack) {
      MzanalyticsLog.p(e.toString() + stack.toString());
    }
    return false;
  }

  ///getManualtrackpage
  static Future<List?> getManualtrackpage() async {
    try {
      return _channel.invokeMethod('getManualtrackpage');
    } catch (e, stack) {
      MzanalyticsLog.p(e.toString() + stack.toString());
    }
    return [];
  }

  ///setEnableLog
  static Future<void> setEnableLog(bool? parameter) async {
    try {
      _channel.invokeMethod('setEnableLog', {'Enable': parameter});
    } catch (e, stack) {
      MzanalyticsLog.p(e.toString() + stack.toString());
    }
  }

  ///enableExceptionService
  static Future<void> enableExceptionService(bool parameter) async {
    try {
      _channel.invokeMethod('enableExceptionService', {'Enable': parameter});
    } catch (e, stack) {
      MzanalyticsLog.p(e.toString() + stack.toString());
    }
  }

  ///trackException
  static Future<void> trackException(Map exception) async {
    try {
      _channel.invokeMethod('trackException', exception);
    } catch (e, stack) {
      MzanalyticsLog.p(e.toString() + stack.toString());
    }
  }

  // ignore: non_constant_identifier_names
  static Future<void> MZNativeErrorTest() async {
    try {
      _channel.invokeMethod('MZNativeErrorTest');
    } catch (e, stack) {
      MzanalyticsLog.p(e.toString() + stack.toString());
    }
  }
}
