import 'dart:async';
import 'package:built_collection/built_collection.dart';
import 'package:flutter/widgets.dart';
import 'dart:io';
import 'package:mzanalytics_mybmw/mzanalytics_ios.dart';
import 'package:mzanalytics_mybmw/mzanalytics_android.dart';
import 'package:mzanalytics_mybmw/mzanalytics_checkout.dart';
import 'package:mzanalytics_mybmw/mzanalytics_log.dart';

class MzanalyticsMybmw {
  ///初始化SDK
  static Future<void> appid(String appid) async {
    try {
      bool ret = await MzanalyticsCheckOut.appidCheck(appid);
      if (ret == true) {
        if (Platform.isIOS) {
          MzanalyticsiOS.appid(appid);
        } else if (Platform.isAndroid) {
          MzanalyticsAndroid.initEvent(appid);
        }
      }
    } catch (e, stack) {
      MzanalyticsLog.p(e.toString() + stack.toString());
    }
  }

  ///页面浏览
  static Future<void> trackPageView(Map parameter) async {
    try {
      bool ret = await MzanalyticsCheckOut.trackPageViewCheck(parameter);
      if (ret == true) {
        if (Platform.isIOS) {
          MzanalyticsiOS.trackPageView(parameter);
        } else if (Platform.isAndroid) {
          MzanalyticsAndroid.trackPageView(parameter);
        }
      }
    } catch (e, stack) {
      MzanalyticsLog.p(e.toString() + stack.toString());
    }
  }

  ///自定义事件
  static Future<void> trackEvent(Map parameter) async {
    try {
      bool ret = await MzanalyticsCheckOut.trackEventCheck(parameter);
      if (ret == true) {
        if (Platform.isIOS) {
          MzanalyticsiOS.trackEvent(parameter);
        } else if (Platform.isAndroid) {
          MzanalyticsAndroid.trackEvent(parameter);
        }
      }
    } catch (e, stack) {
      MzanalyticsLog.p(e.toString() + stack.toString());
    }
  }

  ///高级事件
  static Future<void> trackSiteCustomEvent(Map parameter) async {
    try {
      bool ret = await MzanalyticsCheckOut.trackSiteCustomEventCheck(parameter);
      if (ret == true) {
        if (Platform.isIOS) {
          MzanalyticsiOS.trackSiteCustomEvent(parameter);
        } else if (Platform.isAndroid) {
          MzanalyticsAndroid.trackSiteCustomEvent(parameter);
        }
      }
    } catch (e, stack) {
      MzanalyticsLog.p(e.toString() + stack.toString());
    }
  }

  ///心跳事件
  static Future<void> trackPulse() async {
    try {
      if (Platform.isIOS) {
        MzanalyticsiOS.trackPulse();
      } else if (Platform.isAndroid) {
        MzanalyticsAndroid.trackPulse();
      }
    } catch (e, stack) {
      MzanalyticsLog.p(e.toString() + stack.toString());
    }
  }

  ///获取访客id
  static Future<String> getCid() async {
    try {
      if (Platform.isIOS) {
        String? cid = await MzanalyticsiOS.getCid();
        return cid??"";
      } else if (Platform.isAndroid) {
        String? cid = await MzanalyticsAndroid.getCid();
        return cid??"";
      }
    } catch (e, stack) {
      MzanalyticsLog.p(e.toString() + stack.toString());
      return "";
    }
    return "";
  }

  ///DeepLink
  static Future<void> onDeepLinkReceived(String url) async {
    try {
      if (url is String) {
        if (Platform.isIOS) {
          MzanalyticsiOS.handleOpenURL(url);
        } else if (Platform.isAndroid) {
          MzanalyticsAndroid.handleOpenURL(url);
        }
      }
    } catch (e, stack) {
      MzanalyticsLog.p(e.toString() + stack.toString());
    }
  }

  /// 获取自定义维度cd和自定义指标cm集合
  static Future<Map?> getCdAndCm() async {
    try {
      if (Platform.isAndroid) {
        return MzanalyticsAndroid.getCdAndCm();
      } else if (Platform.isIOS) {
        return MzanalyticsiOS.getCdAndCm();
      }
    } catch (e, stack) {
      MzanalyticsLog.p(e.toString() + stack.toString());
      return null;
    }
    return Map();
  }

  ///设置自定义维度cd
  static Future<void> setCd(String cdKey, String cdValue) async {
    try {
      bool kb = await MzanalyticsCheckOut.setCdCheck(cdKey);
      bool vb = await MzanalyticsCheckOut.setCdCheck(cdValue);

      if (kb && vb) {
        if (Platform.isAndroid) {
          MzanalyticsAndroid.setCd(cdKey, cdValue);
        } else if (Platform.isIOS) {
          MzanalyticsiOS.setCd(cdKey, cdValue);
        }
      }
    } catch (e, stack) {
      MzanalyticsLog.p(e.toString() + stack.toString());
    }
  }

  ///设置自定义指标cm
  static Future<void> setCm(String cmKey, String cmValue) async {
    try {
      bool ret = await MzanalyticsCheckOut.setCmCheck(cmValue);
      if ((cmKey is String) == false) {
        MzanalyticsLog.p('cm 的 Key 必须为String类型');
        return;
      }
      if (ret) {
        if (Platform.isAndroid) {
          MzanalyticsAndroid.setCm(cmKey, cmValue);
        } else if (Platform.isIOS) {
          MzanalyticsiOS.setCm(cmKey, cmValue);
        }
      }
    } catch (e, stack) {
      MzanalyticsLog.p(e.toString() + stack.toString());
    }
  }

  ///删除App缓存中的自定义维度cd
  static Future<void> deleteCd(String cdKey) async {
    try {
      if (Platform.isAndroid) {
        MzanalyticsAndroid.deleteCd(cdKey);
      } else if (Platform.isIOS) {
        MzanalyticsiOS.deleteCd(cdKey);
      }
    } catch (e, stack) {
      MzanalyticsLog.p(e.toString() + stack.toString());
    }
  }

  ///删除App缓存中的自定义指标cm
  static Future<void> deleteCm(String cmKey) async {
    try {
      if (Platform.isAndroid) {
        MzanalyticsAndroid.deleteCm(cmKey);
      } else if (Platform.isIOS) {
        MzanalyticsiOS.deleteCm(cmKey);
      }
    } catch (e, stack) {
      MzanalyticsLog.p(e.toString() + stack.toString());
    }
  }

  ///设置日志开关
  static Future<void> logEnabled(bool enabled) async {
    try {
      if ((enabled is bool) == false) {
        MzanalyticsLog.p('enabled 必须为bool类型');
        return;
      }
      MzanalyticsLog.enabled = enabled;
      if (Platform.isAndroid) {
        MzanalyticsAndroid.setEnableLog(enabled);
      } else if (Platform.isIOS) {
        MzanalyticsiOS.setEnableLog(enabled);
      }
    } catch (e, stack) {
      MzanalyticsLog.p(e.toString() + stack.toString());
    }
  }

  ///启动crash采集开关
  static Future<void> enableExceptionService(bool enabled) async {
    try {
      if ((enabled is bool) == false) {
        MzanalyticsLog.p('enabled 必须为bool类型');
        return;
      }

      if (Platform.isAndroid) {
        MzanalyticsAndroid.enableExceptionService(enabled);
      } else if (Platform.isIOS) {
        MzanalyticsiOS.enableExceptionService(enabled);
      }
    } catch (e, stack) {
      MzanalyticsLog.p(e.toString() + stack.toString());
    }
  }

  ///setCAIDandVersion
  static Future<void> setCAIDandVersion(Map parameter) async {
    try {
      if (Platform.isIOS) {
        MzanalyticsiOS.setCAIDandVersion(parameter);
      } else if (Platform.isAndroid) {
        //目前只有iOS存在此逻辑
      }
    } catch (e, stack) {
      MzanalyticsLog.p(e.toString() + stack.toString());
    }
  }

  ///发送exception
  static Future<void> trackException(Map exception) async {
    try {
      bool ret = await MzanalyticsCheckOut.exceptionCheck(exception);
      if (ret) {
        if (Platform.isAndroid) {
          MzanalyticsAndroid.trackException(exception);
        } else if (Platform.isIOS) {
          MzanalyticsiOS.trackException(exception);
        }
      }
    } catch (e, stack) {
      MzanalyticsLog.p(e.toString() + stack.toString());
    }
  }

  ///isAutoTrackpage
  static Future<bool?> isAutoTrackpage() async {
    try {
      if (Platform.isAndroid) {
        return MzanalyticsAndroid.isAutoTrackpage();
      } else if (Platform.isIOS) {
        return MzanalyticsiOS.isAutoTrackpage();
      }
    } catch (e, stack) {
      MzanalyticsLog.p(e.toString() + stack.toString());
    }
    return false;
  }

  ///getManualtrackpage
  static Future<List?> getManualtrackpage() async {
    try {
      if (Platform.isAndroid) {
        return MzanalyticsAndroid.getManualtrackpage();
      } else if (Platform.isIOS) {
        return MzanalyticsiOS.getManualtrackpage();
      }
    } catch (e, stack) {
      MzanalyticsLog.p(e.toString() + stack.toString());
    }
    return [];
  }

  ///内部测试用例 MZNativeErrorTest
  // ignore: non_constant_identifier_names
  static Future<void> MZNativeErrorTest() async {
    try {
      if (Platform.isAndroid) {
        MzanalyticsAndroid.MZNativeErrorTest();
      } else if (Platform.isIOS) {
        MzanalyticsiOS.MZNativeErrorTest();
      }
    } catch (e, stack) {
      MzanalyticsLog.p(e.toString() + stack.toString());
    }
  }
}

///（可选）导航路由观察者方法
// ignore: camel_case_types
class mz_navigation_history_observer extends NavigatorObserver {
  /// A list of all the past routes
  List<Route<dynamic>> _history = <Route<dynamic>>[];

  /// Gets a clone of the navigation history as an immutable list.
  get history => BuiltList<Route<dynamic>>.from(_history);

  /// Gets the top route in the navigation stack.
  get top => _history.last;

  /// A list of all routes that were popped to reach the current.
  List<Route<dynamic>> _poppedRoutes = <Route<dynamic>>[];

  /// Gets a clone of the popped routes as an immutable list.
  get poppedRoutes => BuiltList<Route<dynamic>>.from(_poppedRoutes);

  /// Gets the next route in the navigation history, which is the most recently popped route.
  get next => _poppedRoutes.last;

  /// A stream that broadcasts whenever the navigation history changes.
  StreamController _historyChangeStreamController =
      StreamController.broadcast();

  /// Accessor to the history change stream.
  get historyChangeStream => _historyChangeStreamController.stream;

  static final mz_navigation_history_observer _singleton =
      mz_navigation_history_observer._internal();
  mz_navigation_history_observer._internal();
  factory mz_navigation_history_observer() {
    return _singleton;
  }

  @override
  void didPop(Route<dynamic> route, Route<dynamic>? previousRoute) {
    _poppedRoutes.add(_history.last);
    _history.removeLast();
    _historyChangeStreamController.add(HistoryChange(
      action: NavigationStackAction.pop,
      newRoute: route,
      oldRoute: previousRoute!,
    ));
  }

  ///打开页面
  @override
  void didPush(Route<dynamic> route, Route<dynamic>? previousRoute) {
    _history.add(route);
    _poppedRoutes.remove(route);
    try {
      MzanalyticsMybmw.isAutoTrackpage().then((bool? isAutoTrackpage) {
        MzanalyticsMybmw.getManualtrackpage().then((List? blackpvList) {
          if (route.settings.name != null &&
              route.settings.name != '/' &&
              isAutoTrackpage! &&
              !blackpvList!.contains(route.settings.name)) {
            print('sending...');
            Map selfparams = new Map();
            MzanalyticsMybmw.getCdAndCm().then((Map? value) {
              print("读取的自定义事件:" + value.toString());
              if (value != null) {
                selfparams = value;
              }
              //{'pageTitle': route.settings.name;CustomDimMetrics:字典}; {'pageTitle': route.settings.name;字典}

              selfparams['dt'] = route.settings.name;
              print('准备发送的数据：' + selfparams.toString());
              MzanalyticsMybmw.trackPageView(selfparams);
            });
          }
        });
      });
    } catch (e, stack) {
      MzanalyticsLog.p(e.toString() + stack.toString());
    }
    _historyChangeStreamController.add(HistoryChange(
      action: NavigationStackAction.push,
      newRoute: route,
      oldRoute: previousRoute!,
    ));
  }

  @override
  void didRemove(Route<dynamic> route, Route<dynamic>? previousRoute) {
    _history.remove(route);
    _historyChangeStreamController.add(HistoryChange(
      action: NavigationStackAction.remove,
      newRoute: route,
      oldRoute: previousRoute!,
    ));
  }

  @override
  void didReplace({Route<dynamic>? newRoute, Route<dynamic>? oldRoute}) {
    int oldRouteIndex = _history.indexOf(oldRoute!);
    _history.replaceRange(oldRouteIndex, oldRouteIndex + 1, [newRoute!]);
    _historyChangeStreamController.add(HistoryChange(
      action: NavigationStackAction.replace,
      newRoute: newRoute,
      oldRoute: oldRoute,
    ));
  }
}

/// A class that contains all data that needs to be broadcasted through the history change stream.
class HistoryChange {
  HistoryChange(
      {required this.action, required this.newRoute, required this.oldRoute});

  final NavigationStackAction action;
  final Route<dynamic> newRoute;
  final Route<dynamic> oldRoute;
}

enum NavigationStackAction { push, pop, remove, replace }
